package common;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

import java.io.*;
import java.lang.reflect.Array;
import java.net.Socket;
import java.util.Arrays;

public class JsonCodec implements Codec {

    private InputStream _in;
    private OutputStream _out;
    private ObjectMapper _mapper;

    public JsonCodec(Socket socket) throws IOException {
        this(socket.getInputStream(), socket.getOutputStream());
    }

    public JsonCodec(InputStream in, OutputStream out) {
        _in = in;
        _out = out;
        _mapper = new ObjectMapper();
        _mapper.enable(SerializationFeature.INDENT_OUTPUT);
    }

    @Override
    public void sendRequest(Request request) {
        try {
            _out.write(_mapper.writeValueAsBytes(request));
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Error on request send");
            System.exit(1);
        }
    }

    @Override
    public void sendResponse(double response) {
        double[] responseArray = new double[1];
        responseArray[0] =  response;
        try {
            _out.write(_mapper.writeValueAsBytes(responseArray));
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Error on params send");
            System.exit(1);
        }
    }

    @Override
    public Request fetchRequest() {
        Request request = null;
        try {
            byte[]  rxbytes = new byte[1024];
            int length = _in.read(rxbytes);
            System.out.println("The request: " + getString(rxbytes, length));
            request = _mapper.readValue(rxbytes, Request.class);
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Error on request receive");
            System.exit(1);
        }
        return request;
    }

    @Override
    public double fetchResponse() {
        try {
            double[] responseArray = _mapper.readValue(_in, double[].class);
            return responseArray[0];
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Error on params receive");
            System.exit(1);
        }
        return 0.0;
    }

    public static String getString(byte[] b, int length) {
        byte[] bShort = Arrays.copyOf(b, length);

        return new String(bShort);
    }
}
