package server;

import common.*;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public class ServerStub {

    private final static int RPC_PORT = 1234;

    private final IRPCProcedures implementation;
    private ServerSocket serverSocket;

    public ServerStub(IRPCProcedures implementation) throws IOException {
        this.implementation = implementation;
        serverSocket = new ServerSocket(RPC_PORT);
    }

    public void run() throws IOException {

        while (true) {
            Codec codec = null;
            Socket socket = serverSocket.accept();
            try {
                //codec = new JsonCodec(socket);
                codec = new BinaryCodec(socket);
            } catch (IOException e) {
                e.printStackTrace();
                System.out.println("Codec can not get streams");
                System.exit(1);
            }

            Request request = codec.fetchRequest();

            ProcedureID procedureID = request.getProcedure();

            double result = 0.0;

            switch (procedureID) {
                case addID:
                    result = implementation.add(request.getParam(0), request.getParam(1));
                    break;

                case subID:
                    result = implementation.sub(request.getParam(0), request.getParam(1));
                    break;

                default:
                    System.out.println("Default");
                    break;
            }

            codec.sendResponse(result);

            socket.close();
            System.out.println("Client processing finished");
        }
    }
}
