package common;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

public class BinaryCodec implements Codec {

    private InputStream _in;
    private OutputStream _out;

    public BinaryCodec(InputStream in, OutputStream out) {
        _in = in;
        _out = out;
    }

    public BinaryCodec(Socket socket) throws IOException {
        this(socket.getInputStream(), socket.getOutputStream());

    }

    private void writeLong(long value) throws IOException {
        byte[] data = new byte[8];
        for (int i = 0; i < 8; ++i) {
            data[i] = (byte)((value >> (8 * i)) & 0xFF);
        }
        _out.write(data,0, 8);
    }

    public long readLong() throws IOException {
        byte[] data = new byte[8];
        if (_in.read(data, 0 , 8) != 8) {
            throw new IOException("Error reading long from input stream!");
        }
        long value = 0;
        for (int i = 0; i < 8; ++i) {
            value |= ((long)data[i]  & 0xFF) << (8 * i);
        }
        return value;
    }

    private void writeDouble(double value) throws IOException {
        writeLong(Double.doubleToLongBits(value));
    }

    private double readDouble() throws IOException {
        return Double.longBitsToDouble(readLong());
    }

    @Override
    public void sendRequest(Request request) {
        try {
            writeLong(request.getProcedure().ordinal());
            for (int i = 0; i < request.paramsSize(); i++) {

            }
            writeDouble(request.getParam(0));
            writeDouble(request.getParam(1));
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Can not send request");
            System.exit(1);
        }
    }

    @Override
    public void sendResponse(double response) {
        try {
            writeDouble(response);
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Can not send params");
            System.exit(1);
        }
    }

    @Override
    public Request fetchRequest() {
        Request request = null;
        try {
            ProcedureID procedureID = ProcedureID.values()[(int)readLong()];
            request = new Request(procedureID, readDouble(), readDouble());

        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Can not fetch request");
            System.exit(1);
        }
        return request;
    }

    @Override
    public double fetchResponse() {
        double response = 0.0;
        try {
            response = readDouble();
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Can not fetch request");
            System.exit(1);
        }
        return response;
    }
}
